# Copyright 2016 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.


import pbr.version

version_info = pbr.version.VersionInfo('oslo.config')



#
# Copyright 2013 Mirantis, Inc.
# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslotest import base

from oslo_config import cfg
from oslo_config import fixture as config


class ConfigTestCase(base.BaseTestCase):

    def _make_fixture(self):
        conf = cfg.ConfigOpts()
        config_fixture = config.Config(conf)
        config_fixture.setUp()
        config_fixture.register_opt(cfg.StrOpt(
            'testing_option', default='initial_value'))
        config_fixture.register_opt(cfg.IntOpt(
            'test2', min=0, default=5))
        config_fixture.register_opt(cfg.StrOpt(
            'test3', choices=['a', 'b'], default='a'))
        return config_fixture

    def test_overridden_value(self):
        f = self._make_fixture()
        self.assertEqual(f.conf.get('testing_option'), 'initial_value')
        f.config(testing_option='changed_value')
        self.assertEqual('changed_value',
                         f.conf.get('testing_option'))

    def test_overridden_value_with_wrong_type(self):
        f = self._make_fixture()
        self.assertEqual(5, f.conf.get('test2'))
        self.assertEqual('a', f.conf.get('test3'))
        # enforce type will always be true now
        self.assertRaises(ValueError, f.config, test2=-1)
        self.assertRaises(ValueError, f.config, test3='c')

    def test_cleanup(self):
        f = self._make_fixture()
        f.config(testing_option='changed_value')
        self.assertEqual(f.conf.get('testing_option'),
                         'changed_value')
        f.conf.reset()
        self.assertEqual(f.conf.get('testing_option'), 'initial_value')

    def test_register_option(self):
        f = self._make_fixture()
        opt = cfg.StrOpt('new_test_opt', default='initial_value')
        f.register_opt(opt)
        self.assertEqual(f.conf.get('new_test_opt'),
                         opt.default)

    def test_register_options(self):
        f = self._make_fixture()
        opt1 = cfg.StrOpt('first_test_opt', default='initial_value_1')
        opt2 = cfg.StrOpt('second_test_opt', default='initial_value_2')
        f.register_opts([opt1, opt2])
        self.assertEqual(f.conf.get('first_test_opt'), opt1.default)
        self.assertEqual(f.conf.get('second_test_opt'), opt2.default)

    def test_cleanup_unregister_option(self):
        f = self._make_fixture()
        opt = cfg.StrOpt('new_test_opt', default='initial_value')
        f.register_opt(opt)
        self.assertEqual(f.conf.get('new_test_opt'),
                         opt.default)
        f.cleanUp()
        self.assertRaises(cfg.NoSuchOptError, f.conf.get, 'new_test_opt')

    def test_register_cli_option(self):
        f = self._make_fixture()
        opt = cfg.StrOpt('new_test_opt', default='initial_value')
        f.register_cli_opt(opt)
        self.assertEqual(f.conf.get('new_test_opt'),
                         opt.default)

    def test_register_cli_options(self):
        f = self._make_fixture()
        opt1 = cfg.StrOpt('first_test_opt', default='initial_value_1')
        opt2 = cfg.StrOpt('second_test_opt', default='initial_value_2')
        f.register_cli_opts([opt1, opt2])
        self.assertEqual(f.conf.get('first_test_opt'), opt1.default)
        self.assertEqual(f.conf.get('second_test_opt'), opt2.default)

    def test_cleanup_unregister_cli_option(self):
        f = self._make_fixture()
        opt = cfg.StrOpt('new_test_opt', default='initial_value')
        f.register_cli_opt(opt)
        self.assertEqual(f.conf.get('new_test_opt'),
                         opt.default)
        f.cleanUp()
        self.assertRaises(cfg.NoSuchOptError, f.conf.get, 'new_test_opt')

    def test_load_raw_values(self):
        f = self._make_fixture()
        f.load_raw_values(first_test_opt='loaded_value_1',
                          second_test_opt='loaded_value_2')

        # Must not be registered.
        self.assertRaises(cfg.NoSuchOptError, f.conf.get, 'first_test_opt')
        self.assertRaises(cfg.NoSuchOptError, f.conf.get, 'second_test_opt')

        opt1 = cfg.StrOpt('first_test_opt', default='initial_value_1')
        opt2 = cfg.StrOpt('second_test_opt', default='initial_value_2')

        f.register_opt(opt1)
        f.register_opt(opt2)

        self.assertEqual(f.conf.first_test_opt, 'loaded_value_1')
        self.assertEqual(f.conf.second_test_opt, 'loaded_value_2')

        # Cleanup.
        f.cleanUp()

        # Must no longer be registered.
        self.assertRaises(cfg.NoSuchOptError, f.conf.get, 'first_test_opt')
        self.assertRaises(cfg.NoSuchOptError, f.conf.get, 'second_test_opt')

        # Even when registered, must be default.
        f.register_opt(opt1)
        f.register_opt(opt2)
        self.assertEqual(f.conf.first_test_opt, 'initial_value_1')
        self.assertEqual(f.conf.second_test_opt, 'initial_value_2')

    def test_assert_default_files_cleanup(self):
        """Assert that using the fixture forces a clean list."""
        f = self._make_fixture()
        self.assertNotIn('default_config_files', f.conf)
        self.assertNotIn('default_config_dirs', f.conf)

        config_files = ['./test_fixture.conf']
        config_dirs = ['./test_fixture.conf.d']
        f.set_config_files(config_files)
        f.set_config_dirs(config_dirs)

        self.assertEqual(f.conf.default_config_files, config_files)
        self.assertEqual(f.conf.default_config_dirs, config_dirs)
        f.cleanUp()

        self.assertNotIn('default_config_files', f.conf)
        self.assertNotIn('default_config_dirs', f.conf)

    def test_load_custom_files(self):
        f = self._make_fixture()
        self.assertNotIn('default_config_files', f.conf)
        config_files = ['./oslo_config/tests/test_fixture.conf']
        f.set_config_files(config_files)

        opt1 = cfg.StrOpt('first_test_opt', default='initial_value_1')
        opt2 = cfg.StrOpt('second_test_opt', default='initial_value_2')

        f.register_opt(opt1)
        f.register_opt(opt2)

        self.assertEqual('loaded_value_1', f.conf.get('first_test_opt'))
        self.assertEqual('loaded_value_2', f.conf.get('second_test_opt'))

    def test_set_default(self):
        f = self._make_fixture()
        opt = cfg.StrOpt('new_test_opt', default='initial_value')
        # Register the option directly so it is not cleaned up by the
        # fixture.
        f.conf.register_opt(opt)
        f.set_default(
            name='new_test_opt',
            default='alternate_value',
        )
        self.assertEqual('alternate_value', f.conf.new_test_opt)
        f.cleanUp()
        self.assertEqual('initial_value', f.conf.new_test_opt)

    def test_set_default_group(self):
        f = self._make_fixture()
        opt = cfg.StrOpt('new_test_opt', default='initial_value')
        # Register the option directly so it is not cleaned up by the
        # fixture.
        f.conf.register_opt(opt, group='foo')
        f.set_default(
            name='new_test_opt',
            default='alternate_value',
            group='foo',
        )
        self.assertEqual('alternate_value', f.conf.foo.new_test_opt)
        f.cleanUp()
        self.assertEqual('initial_value', f.conf.foo.new_test_opt)



# Copyright 2014 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslotest import base as test_base
import testtools

from oslo_config import cfg
from oslo_config import cfgfilter


class BaseTestCase(test_base.BaseTestCase):

    def setUp(self, conf=None):
        super(BaseTestCase, self).setUp()
        if conf is None:
            self.conf = cfg.ConfigOpts()
        else:
            self.conf = conf
        self.fconf = cfgfilter.ConfigFilter(self.conf)


class RegisterTestCase(BaseTestCase):

    def test_register_opt_default(self):
        self.fconf.register_opt(cfg.StrOpt('foo', default='bar'))

        self.assertEqual('bar', self.fconf.foo)
        self.assertEqual('bar', self.fconf['foo'])
        self.assertIn('foo', self.fconf)
        self.assertEqual(['foo'], list(self.fconf))
        self.assertEqual(1, len(self.fconf))

        self.assertNotIn('foo', self.conf)
        self.assertEqual(0, len(self.conf))
        self.assertRaises(cfg.NoSuchOptError, getattr, self.conf, 'foo')

    def test_register_opt_none_default(self):
        self.fconf.register_opt(cfg.StrOpt('foo'))

        self.assertIsNone(self.fconf.foo)
        self.assertIsNone(self.fconf['foo'])
        self.assertIn('foo', self.fconf)
        self.assertEqual(['foo'], list(self.fconf))
        self.assertEqual(1, len(self.fconf))

        self.assertNotIn('foo', self.conf)
        self.assertEqual(0, len(self.conf))
        self.assertRaises(cfg.NoSuchOptError, getattr, self.conf, 'foo')

    def test_register_grouped_opt_default(self):
        self.fconf.register_opt(cfg.StrOpt('foo', default='bar'),
                                group='blaa')

        self.assertEqual('bar', self.fconf.blaa.foo)
        self.assertEqual('bar', self.fconf['blaa']['foo'])
        self.assertIn('blaa', self.fconf)
        self.assertIn('foo', self.fconf.blaa)
        self.assertEqual(['blaa'], list(self.fconf))
        self.assertEqual(['foo'], list(self.fconf.blaa))
        self.assertEqual(1, len(self.fconf))
        self.assertEqual(1, len(self.fconf.blaa))

        self.assertNotIn('blaa', self.conf)
        self.assertEqual(0, len(self.conf))
        self.assertRaises(cfg.NoSuchOptError, getattr, self.conf, 'blaa')

    def test_register_grouped_opt_none_default(self):
        self.fconf.register_opt(cfg.StrOpt('foo'), group='blaa')

        self.assertIsNone(self.fconf.blaa.foo)
        self.assertIsNone(self.fconf['blaa']['foo'])
        self.assertIn('blaa', self.fconf)
        self.assertIn('foo', self.fconf.blaa)
        self.assertEqual(['blaa'], list(self.fconf))
        self.assertEqual(['foo'], list(self.fconf.blaa))
        self.assertEqual(1, len(self.fconf))
        self.assertEqual(1, len(self.fconf.blaa))

        self.assertNotIn('blaa', self.conf)
        self.assertEqual(0, len(self.conf))
        self.assertRaises(cfg.NoSuchOptError, getattr, self.conf, 'blaa')

    def test_register_group(self):
        group = cfg.OptGroup('blaa')
        self.fconf.register_group(group)
        self.fconf.register_opt(cfg.StrOpt('foo'), group=group)

        self.assertIsNone(self.fconf.blaa.foo)
        self.assertIsNone(self.fconf['blaa']['foo'])
        self.assertIn('blaa', self.fconf)
        self.assertIn('foo', self.fconf.blaa)
        self.assertEqual(['blaa'], list(self.fconf))
        self.assertEqual(['foo'], list(self.fconf.blaa))
        self.assertEqual(1, len(self.fconf))
        self.assertEqual(1, len(self.fconf.blaa))

        self.assertNotIn('blaa', self.conf)
        self.assertEqual(0, len(self.conf))
        self.assertRaises(cfg.NoSuchOptError, getattr, self.conf, 'blaa')

    def test_register_opts(self):
        self.fconf.register_opts([cfg.StrOpt('foo'),
                                  cfg.StrOpt('bar')])
        self.assertIn('foo', self.fconf)
        self.assertIn('bar', self.fconf)
        self.assertNotIn('foo', self.conf)
        self.assertNotIn('bar', self.conf)

    def test_register_known_cli_opt(self):
        self.conf.register_opt(cfg.StrOpt('foo'))
        self.fconf.register_cli_opt(cfg.StrOpt('foo'))
        self.assertIn('foo', self.fconf)
        self.assertIn('foo', self.conf)

    def test_register_unknown_cli_opt(self):
        with testtools.ExpectedException(cfgfilter.CliOptRegisteredError):
            self.fconf.register_cli_opt(cfg.StrOpt('foo'))

    def test_register_known_cli_opts(self):
        self.conf.register_cli_opts([cfg.StrOpt('foo'), cfg.StrOpt('bar')])
        self.fconf.register_cli_opts([cfg.StrOpt('foo'), cfg.StrOpt('bar')])
        self.assertIn('foo', self.fconf)
        self.assertIn('bar', self.fconf)
        self.assertIn('foo', self.conf)
        self.assertIn('bar', self.conf)

    def test_register_unknown_cli_opts(self):
        self.conf.register_cli_opt(cfg.StrOpt('foo'))
        with testtools.ExpectedException(cfgfilter.CliOptRegisteredError):
            self.fconf.register_cli_opts([
                cfg.StrOpt('foo'),
                cfg.StrOpt('bar')
            ])

    def test_register_opts_grouped(self):
        self.fconf.register_opts([cfg.StrOpt('foo'), cfg.StrOpt('bar')],
                                 group='blaa')
        self.assertIn('foo', self.fconf.blaa)
        self.assertIn('bar', self.fconf.blaa)
        self.assertNotIn('blaa', self.conf)

    def test_register_known_cli_opt_grouped(self):
        self.conf.register_cli_opt(cfg.StrOpt('foo'), group='blaa')
        self.fconf.register_cli_opt(cfg.StrOpt('foo'), group='blaa')
        self.assertIn('foo', self.fconf.blaa)
        self.assertIn('blaa', self.fconf)
        self.assertIn('blaa', self.conf)

    def test_register_unknown_cli_opt_grouped(self):
        with testtools.ExpectedException(cfgfilter.CliOptRegisteredError):
            self.fconf.register_cli_opt(cfg.StrOpt('foo'), group='blaa')

    def test_register_known_cli_opts_grouped(self):
        self.conf.register_cli_opts([cfg.StrOpt('foo'), cfg.StrOpt('bar')],
                                    group='blaa')
        self.fconf.register_cli_opts([cfg.StrOpt('foo'), cfg.StrOpt('bar')],
                                     group='blaa')
        self.assertIn('foo', self.fconf.blaa)
        self.assertIn('bar', self.fconf.blaa)
        self.assertIn('blaa', self.fconf)
        self.assertIn('blaa', self.conf)

    def test_register_unknown_opts_grouped(self):
        self.conf.register_cli_opts([cfg.StrOpt('bar')], group='blaa')
        with testtools.ExpectedException(cfgfilter.CliOptRegisteredError):
            self.fconf.register_cli_opts([
                cfg.StrOpt('foo'),
                cfg.StrOpt('bar')
            ], group='blaa')

    def test_unknown_opt(self):
        self.assertNotIn('foo', self.fconf)
        self.assertEqual(0, len(self.fconf))
        self.assertRaises(cfg.NoSuchOptError, getattr, self.fconf, 'foo')
        self.assertNotIn('blaa', self.conf)

    def test_blocked_opt(self):
        self.conf.register_opt(cfg.StrOpt('foo'))

        self.assertIn('foo', self.conf)
        self.assertEqual(1, len(self.conf))
        self.assertIsNone(self.conf.foo)
        self.assertNotIn('foo', self.fconf)
        self.assertEqual(0, len(self.fconf))
        self.assertRaises(cfg.NoSuchOptError, getattr, self.fconf, 'foo')

    def test_already_registered_opt(self):
        self.conf.register_opt(cfg.StrOpt('foo'))
        self.fconf.register_opt(cfg.StrOpt('foo'))

        self.assertIn('foo', self.conf)
        self.assertEqual(1, len(self.conf))
        self.assertIsNone(self.conf.foo)
        self.assertIn('foo', self.fconf)
        self.assertEqual(1, len(self.fconf))
        self.assertIsNone(self.fconf.foo)

        self.conf.set_override('foo', 'bar')

        self.assertEqual('bar', self.conf.foo)
        self.assertEqual('bar', self.fconf.foo)

    def test_already_registered_opts(self):
        self.conf.register_opts([cfg.StrOpt('foo'),
                                 cfg.StrOpt('fu')])
        self.fconf.register_opts([cfg.StrOpt('foo'),
                                  cfg.StrOpt('bu')])

        self.assertIn('foo', self.conf)
        self.assertIn('fu', self.conf)
        self.assertNotIn('bu', self.conf)
        self.assertEqual(2, len(self.conf))
        self.assertIsNone(self.conf.foo)
        self.assertIsNone(self.conf.fu)
        self.assertIn('foo', self.fconf)
        self.assertIn('bu', self.fconf)
        self.assertNotIn('fu', self.fconf)
        self.assertEqual(2, len(self.fconf))
        self.assertIsNone(self.fconf.foo)
        self.assertIsNone(self.fconf.bu)

        self.conf.set_override('foo', 'bar')

        self.assertEqual('bar', self.conf.foo)
        self.assertEqual('bar', self.fconf.foo)

    def test_already_registered_cli_opt(self):
        self.conf.register_cli_opt(cfg.StrOpt('foo'))
        self.fconf.register_cli_opt(cfg.StrOpt('foo'))

        self.assertIn('foo', self.conf)
        self.assertEqual(1, len(self.conf))
        self.assertIsNone(self.conf.foo)
        self.assertIn('foo', self.fconf)
        self.assertEqual(1, len(self.fconf))
        self.assertIsNone(self.fconf.foo)

        self.conf.set_override('foo', 'bar')

        self.assertEqual('bar', self.conf.foo)
        self.assertEqual('bar', self.fconf.foo)

    def test_already_registered_cli_opts(self):
        self.conf.register_cli_opts([cfg.StrOpt('foo'),
                                     cfg.StrOpt('fu')])
        self.fconf.register_cli_opts([cfg.StrOpt('foo'),
                                      cfg.StrOpt('fu')])

        self.assertIn('foo', self.conf)
        self.assertIn('fu', self.conf)
        self.assertEqual(2, len(self.conf))
        self.assertIsNone(self.conf.foo)
        self.assertIsNone(self.conf.fu)
        self.assertIn('foo', self.fconf)
        self.assertIn('fu', self.fconf)
        self.assertEqual(2, len(self.fconf))
        self.assertIsNone(self.fconf.foo)
        self.assertIsNone(self.fconf.fu)

        self.conf.set_override('foo', 'bar')

        self.assertEqual('bar', self.conf.foo)
        self.assertEqual('bar', self.fconf.foo)


class ImportTestCase(BaseTestCase):

    def setUp(self):
        super(ImportTestCase, self).setUp(cfg.CONF)

    def test_import_opt(self):
        self.assertFalse(hasattr(self.conf, 'fblaa'))
        self.conf.import_opt('fblaa', 'oslo_config.tests.testmods.fblaa_opt')
        self.assertTrue(hasattr(self.conf, 'fblaa'))
        self.assertFalse(hasattr(self.fconf, 'fblaa'))
        self.fconf.import_opt('fblaa', 'oslo_config.tests.testmods.fblaa_opt')
        self.assertTrue(hasattr(self.fconf, 'fblaa'))

    def test_import_opt_in_group(self):
        self.assertFalse(hasattr(self.conf, 'fbar'))
        self.conf.import_opt('foo', 'oslo_config.tests.testmods.fbar_foo_opt',
                             group='fbar')
        self.assertTrue(hasattr(self.conf, 'fbar'))
        self.assertTrue(hasattr(self.conf.fbar, 'foo'))
        self.assertFalse(hasattr(self.fconf, 'fbar'))
        self.fconf.import_opt('foo', 'oslo_config.tests.testmods.fbar_foo_opt',
                              group='fbar')
        self.assertTrue(hasattr(self.fconf, 'fbar'))
        self.assertTrue(hasattr(self.fconf.fbar, 'foo'))

    def test_import_group(self):
        self.assertFalse(hasattr(self.conf, 'fbaar'))
        self.conf.import_group('fbaar',
                               'oslo_config.tests.testmods.fbaar_baa_opt')
        self.assertTrue(hasattr(self.conf, 'fbaar'))
        self.assertTrue(hasattr(self.conf.fbaar, 'baa'))
        self.assertFalse(hasattr(self.fconf, 'fbaar'))
        self.fconf.import_group('fbaar',
                                'oslo_config.tests.testmods.fbaar_baa_opt')
        self.assertTrue(hasattr(self.fconf, 'fbaar'))
        self.assertTrue(hasattr(self.fconf.fbaar, 'baa'))


class ExposeTestCase(BaseTestCase):

    def test_expose_opt(self):
        self.assertFalse(hasattr(self.conf, 'foo'))
        self.assertFalse(hasattr(self.fconf, 'foo'))

        self.conf.register_opt(cfg.StrOpt('foo'))
        self.conf.set_override('foo', 'bar')

        self.assertTrue(hasattr(self.conf, 'foo'))
        self.assertEqual('bar', self.conf.foo)
        self.assertFalse(hasattr(self.fconf, 'foo'))

        self.fconf.expose_opt('foo')
        self.assertTrue(hasattr(self.conf, 'foo'))
        self.assertTrue(hasattr(self.fconf, 'foo'))
        self.assertEqual('bar', self.fconf.foo)

    def test_expose_opt_with_group(self):
        self.assertFalse(hasattr(self.conf, 'foo'))
        self.assertFalse(hasattr(self.fconf, 'foo'))

        self.conf.register_opt(cfg.StrOpt('foo'), group='group')
        self.conf.set_override('foo', 'bar', group='group')

        self.assertTrue(hasattr(self.conf.group, 'foo'))
        self.assertEqual('bar', self.conf.group.foo)
        self.assertFalse(hasattr(self.fconf, 'group'))

        self.fconf.expose_opt('foo', group='group')
        self.assertTrue(hasattr(self.conf.group, 'foo'))
        self.assertTrue(hasattr(self.fconf.group, 'foo'))
        self.assertEqual('bar', self.fconf.group.foo)

    def test_expose_group(self):
        self.conf.register_opts([cfg.StrOpt('foo'),
                                 cfg.StrOpt('bar')], group='group')
        self.conf.register_opts([cfg.StrOpt('foo'),
                                 cfg.StrOpt('bar')], group='another')
        self.conf.set_override('foo', 'a', group='group')
        self.conf.set_override('bar', 'b', group='group')

        self.fconf.expose_group('group')

        self.assertEqual('a', self.fconf.group.foo)
        self.assertEqual('b', self.fconf.group.bar)
        self.assertFalse(hasattr(self.fconf, 'another'))
        self.assertTrue(hasattr(self.conf, 'another'))
